@@ -18,6 +18,7 @@
 #include <stdio.h>
 #include <string.h>
 
+#include "environ.h"
 #include "connection.h"
 #include "statement.h"
 #include "qresult.h"
@@ -39,7 +40,7 @@ PGAPI_Prepare(HSTMT hstmt,
 	CSTR func = "PGAPI_Prepare";
 	StatementClass *self = (StatementClass *) hstmt;
 
-	mylog("%s: entering... len=%d\n", func, cbSqlStr);
+	mylog("%s: entering...\n", func);
 
 	if (!self)
 	{
@@ -101,7 +102,7 @@ PGAPI_Prepare(HSTMT hstmt,
 	if (!szSqlStr[0])
 		self->statement = strdup("");
 	else
-		self->statement = make_string(szSqlStr, cbSqlStr, NULL);
+		self->statement = make_string(szSqlStr, cbSqlStr, NULL, 0);
 	if (!self->statement)
 	{
 		SC_set_error(self, STMT_NO_MEMORY_ERROR, "No memory available to store statement");
@@ -130,7 +131,8 @@ RETCODE		SQL_API
 PGAPI_ExecDirect(
 				 HSTMT hstmt,
 				 UCHAR FAR * szSqlStr,
-				 SDWORD cbSqlStr)
+				 SDWORD cbSqlStr,
+				 UWORD flag)
 {
 	StatementClass *stmt = (StatementClass *) hstmt;
 	RETCODE		result;
@@ -138,19 +140,14 @@ PGAPI_ExecDirect(
 
 	mylog("%s: entering...\n", func);
 
-	if (!stmt)
-	{
-		SC_log_error(func, "", NULL);
-		return SQL_INVALID_HANDLE;
-	}
-
-	SC_initialize_stmts(stmt, TRUE);
+	if (result = SC_initialize_and_recycle(stmt), SQL_SUCCESS != result)
+		return result;
 
 	/*
 	 * keep a copy of the un-parametrized statement, in case they try to
 	 * execute this statement again
 	 */
-	stmt->statement = make_string(szSqlStr, cbSqlStr, NULL);
+	stmt->statement = make_string(szSqlStr, cbSqlStr, NULL, 0);
 	if (!stmt->statement)
 	{
 		SC_set_error(stmt, STMT_NO_MEMORY_ERROR, "No memory available to store statement");
@@ -160,9 +157,6 @@ PGAPI_ExecDirect(
 
 	mylog("**** %s: hstmt=%u, statement='%s'\n", func, hstmt, stmt->statement);
 
-	stmt->prepare = FALSE;
-	SC_set_prepared(stmt, FALSE);
-
 	/*
 	 * If an SQLPrepare was performed prior to this, but was left in the
 	 * premature state because an error occurred prior to SQLExecute then
@@ -183,7 +177,7 @@ PGAPI_ExecDirect(
 
 	mylog("%s: calling PGAPI_Execute...\n", func);
 
-	result = PGAPI_Execute(hstmt);
+	result = PGAPI_Execute(hstmt, flag);
 
 	mylog("%s: returned %hd from PGAPI_Execute\n", func, result);
 	return result;
@@ -293,8 +287,6 @@ RETCODE	Exec_with_parameters_resolved(StatementClass *stmt, BOOL *exec_end)
 
 		if (kres = res->next, kres)
 		{
-			if (kres->fields)
-				CI_Destructor(kres->fields);
 			kres->fields = res->fields;
 			res->fields = NULL;
 			kres->num_fields = res->num_fields;
@@ -325,8 +317,8 @@ RETCODE	Exec_with_parameters_resolved(StatementClass *stmt, BOOL *exec_end)
 			stmt->execute_statement = NULL;
 		}
 	}
-#if (ODBCVER >= 0x0300)
-	ipdopts = SC_get_IPD(stmt);
+
+	ipdopts = SC_get_IPDF(stmt);
 	if (ipdopts->param_status_ptr)
 	{
 		switch (retval)
@@ -342,10 +334,10 @@ RETCODE	Exec_with_parameters_resolved(StatementClass *stmt, BOOL *exec_end)
 				break;
 		}
 	}
-#endif /* ODBCVER */
+
 	if (end_row = stmt->exec_end_row, end_row < 0)
 	{
-		apdopts = SC_get_APD(stmt);
+		apdopts = SC_get_APDF(stmt);
 		end_row = apdopts->paramset_size - 1;
 	}
 	if (stmt->inaccurate_result ||
@@ -357,7 +349,27 @@ RETCODE	Exec_with_parameters_resolved(StatementClass *stmt, BOOL *exec_end)
 	else
 		stmt->exec_current_row++;
 	if (res)
+	{
+
+        	EnvironmentClass *env = (EnvironmentClass *) (conn->henv);
+		const char *cmd = QR_get_command(res);
+
+		if (retval == SQL_SUCCESS && cmd && EN_is_odbc3(env))
+		{
+			int	count;
+
+			if (sscanf(cmd , "UPDATE %d", &count) == 1)
+				;
+			else if (sscanf(cmd , "DELETE %d", &count) == 1)
+				;
+			else
+				count = -1;
+			if (0 == count)
+				retval = SQL_NO_DATA;
+		}
+
 		stmt->diag_row_count = res->recent_processed_row_count;
+	}
 	/*
 	 *	The cursor's info was changed ?
 	 */
@@ -373,8 +385,7 @@ RETCODE	Exec_with_parameters_resolved(StatementClass *stmt, BOOL *exec_end)
 
 /*	Execute a prepared SQL statement */
 RETCODE		SQL_API
-PGAPI_Execute(
-			  HSTMT hstmt)
+PGAPI_Execute(HSTMT hstmt, UWORD flag)
 {
 	CSTR func = "PGAPI_Execute";
 	StatementClass *stmt = (StatementClass *) hstmt;
@@ -393,7 +404,7 @@ PGAPI_Execute(
 		return SQL_INVALID_HANDLE;
 	}
 
-	apdopts = SC_get_APD(stmt);
+	apdopts = SC_get_APDF(stmt);
 	/*
 	 * If the statement is premature, it means we already executed it from
 	 * an SQLPrepare/SQLDescribeCol type of scenario.  So just return
@@ -487,12 +498,12 @@ PGAPI_Execute(
 		end_row = apdopts->paramset_size - 1; 
 	if (stmt->exec_current_row < 0)
 		stmt->exec_current_row = start_row;
-	ipdopts = SC_get_IPD(stmt);
+	ipdopts = SC_get_IPDF(stmt);
 	if (stmt->exec_current_row == start_row)
 	{
 		if (ipdopts->param_processed_ptr)
 			*ipdopts->param_processed_ptr = 0;
-#if (ODBCVER >= 0x0300)
+
 		/*
 	 	 *	Initialize the param_status_ptr 
 	 	 */
@@ -501,13 +512,13 @@ PGAPI_Execute(
 			for (i = 0; i <= end_row; i++)
 				ipdopts->param_status_ptr[i] = SQL_PARAM_UNUSED;
 		}
-#endif /* ODBCVER */
+
 		if (recycle && !recycled)
 			SC_recycle_statement(stmt);
 	}
 
 next_param_row:
-#if (ODBCVER >= 0x0300)
+
 	if (apdopts->param_operation_ptr)
 	{
 		while (apdopts->param_operation_ptr[stmt->exec_current_row] == SQL_PARAM_IGNORE)
@@ -525,7 +536,7 @@ next_param_row:
 	 */
 	if (ipdopts->param_status_ptr)
 		ipdopts->param_status_ptr[stmt->exec_current_row] = SQL_PARAM_ERROR;
-#endif /* ODBCVER */
+
 	/*
 	 * Check if statement has any data-at-execute parameters when it is
 	 * not in SC_pre_execute.
@@ -748,6 +759,7 @@ PGAPI_Cancel(
 	stmt->data_at_exec = -1;
 	stmt->current_exec_param = -1;
 	stmt->put_data = FALSE;
+	cancelNeedDataState(stmt);
 
 	return SQL_SUCCESS;
 }
@@ -775,7 +787,7 @@ PGAPI_NativeSql(
 
 	mylog("%s: entering...cbSqlStrIn=%d\n", func, cbSqlStrIn);
 
-	ptr = (cbSqlStrIn == 0) ? "" : make_string(szSqlStrIn, cbSqlStrIn, NULL);
+	ptr = (cbSqlStrIn == 0) ? "" : make_string(szSqlStrIn, cbSqlStrIn, NULL, 0);
 	if (!ptr)
 	{
 		CC_set_error(conn, CONN_NO_MEMORY_ERROR, "No memory available to store native sql string");
@@ -817,11 +829,11 @@ PGAPI_ParamData(
 				PTR FAR * prgbValue)
 {
 	CSTR func = "PGAPI_ParamData";
-	StatementClass *stmt = (StatementClass *) hstmt;
+	StatementClass *stmt = (StatementClass *) hstmt, *estmt;
 	APDFields	*apdopts;
 	IPDFields	*ipdopts;
-	int			i,
-				retval;
+	RETCODE		retval;
+	int		i;
 	ConnInfo   *ci;
 
 	mylog("%s: entering...\n", func);
@@ -832,18 +844,19 @@ PGAPI_ParamData(
 		return SQL_INVALID_HANDLE;
 	}
 	ci = &(SC_get_conn(stmt)->connInfo);
-	apdopts = SC_get_APD(stmt);
 
-	mylog("%s: data_at_exec=%d, params_alloc=%d\n", func, stmt->data_at_exec, apdopts->allocated);
+	estmt = stmt->execute_delegate ? stmt->execute_delegate : stmt;
+	apdopts = SC_get_APDF(estmt);
+	mylog("%s: data_at_exec=%d, params_alloc=%d\n", func, estmt->data_at_exec, apdopts->allocated);
 
-	if (stmt->data_at_exec < 0)
+	if (estmt->data_at_exec < 0)
 	{
 		SC_set_error(stmt, STMT_SEQUENCE_ERROR, "No execution-time parameters for this statement");
 		SC_log_error(func, "", stmt);
 		return SQL_ERROR;
 	}
 
-	if (stmt->data_at_exec > apdopts->allocated)
+	if (estmt->data_at_exec > apdopts->allocated)
 	{
 		SC_set_error(stmt, STMT_SEQUENCE_ERROR, "Too many execution-time parameters were present");
 		SC_log_error(func, "", stmt);
@@ -851,33 +864,36 @@ PGAPI_ParamData(
 	}
 
 	/* close the large object */
-	if (stmt->lobj_fd >= 0)
+	if (estmt->lobj_fd >= 0)
 	{
-		lo_close(stmt->hdbc, stmt->lobj_fd);
+		lo_close(estmt->hdbc, estmt->lobj_fd);
 
 		/* commit transaction if needed */
-		if (!ci->drivers.use_declarefetch && CC_is_in_autocommit(stmt->hdbc))
+		if (!ci->drivers.use_declarefetch && CC_is_in_autocommit(estmt->hdbc))
 		{
-			if (!CC_commit(stmt->hdbc))
+			if (!CC_commit(estmt->hdbc))
 			{
 				SC_set_error(stmt, STMT_EXEC_ERROR, "Could not commit (in-line) a transaction");
 				SC_log_error(func, "", stmt);
 				return SQL_ERROR;
 			}
 		}
-		stmt->lobj_fd = -1;
+		estmt->lobj_fd = -1;
 	}
 
 	/* Done, now copy the params and then execute the statement */
-	ipdopts = SC_get_IPD(stmt);
-	if (stmt->data_at_exec == 0)
+	ipdopts = SC_get_IPDF(estmt);
+	if (estmt->data_at_exec == 0)
 	{
 		BOOL	exec_end;
 
-		retval = Exec_with_parameters_resolved(stmt, &exec_end);
+		retval = Exec_with_parameters_resolved(estmt, &exec_end);
 		if (exec_end)
-			return retval;
-		if (retval = PGAPI_Execute(stmt), SQL_NEED_DATA != retval)
+		{
+			stmt->execute_delegate = NULL;
+			return dequeueNeedDataCallback(retval, stmt);
+		}
+		if (retval = PGAPI_Execute(estmt, 0), SQL_NEED_DATA != retval)
 			return retval;
 	}
 
@@ -885,17 +901,29 @@ PGAPI_ParamData(
 	 * Set beginning param;  if first time SQLParamData is called , start
 	 * at 0. Otherwise, start at the last parameter + 1.
 	 */
-	i = stmt->current_exec_param >= 0 ? stmt->current_exec_param + 1 : 0;
+	i = estmt->current_exec_param >= 0 ? estmt->current_exec_param + 1 : 0;
 
 	/* At least 1 data at execution parameter, so Fill in the token value */
 	for (; i < apdopts->allocated; i++)
 	{
 		if (apdopts->parameters[i].data_at_exec)
 		{
-			stmt->data_at_exec--;
-			stmt->current_exec_param = i;
-			stmt->put_data = FALSE;
-			*prgbValue = apdopts->parameters[i].buffer;	/* token */
+			estmt->data_at_exec--;
+			estmt->current_exec_param = i;
+			estmt->put_data = FALSE;
+			if (prgbValue)
+			{
+				/* returns token here */
+				if (stmt->execute_delegate)
+				{
+					UInt4	offset = apdopts->param_offset_ptr ? *apdopts->param_offset_ptr : 0;
+					UInt4	perrow = apdopts->param_bind_type > 0 ? apdopts->param_bind_type : apdopts->parameters[i].buflen; 
+
+					*prgbValue = apdopts->parameters[i].buffer + offset + estmt->exec_current_row * perrow;
+				}
+				else
+					*prgbValue = apdopts->parameters[i].buffer;
+			}
 			break;
 		}
 	}
@@ -915,17 +943,19 @@ PGAPI_PutData(
 			  SDWORD cbValue)
 {
 	CSTR func = "PGAPI_PutData";
-	StatementClass *stmt = (StatementClass *) hstmt;
+	StatementClass *stmt = (StatementClass *) hstmt, *estmt;
 	ConnectionClass *conn;
 	APDFields	*apdopts;
 	IPDFields	*ipdopts;
+	PutDataInfo	*pdata;
 	int			old_pos,
 				retval;
 	ParameterInfoClass *current_param;
 	ParameterImplClass *current_iparam;
+	PutDataClass	*current_pdata;
 	char	   *buffer, *putbuf, *allocbuf = NULL;
 	Int2		ctype;
-	SDWORD		putlen;
+	SDWORD          putlen;
 	BOOL		lenset = FALSE;
 
 	mylog("%s: entering...\n", func);
@@ -936,32 +966,33 @@ PGAPI_PutData(
 		return SQL_INVALID_HANDLE;
 	}
 
-	apdopts = SC_get_APD(stmt);
-	if (stmt->current_exec_param < 0)
+	estmt = stmt->execute_delegate ? stmt->execute_delegate : stmt;
+	apdopts = SC_get_APDF(estmt);
+	if (estmt->current_exec_param < 0)
 	{
 		SC_set_error(stmt, STMT_SEQUENCE_ERROR, "Previous call was not SQLPutData or SQLParamData");
 		SC_log_error(func, "", stmt);
 		return SQL_ERROR;
 	}
 
-	current_param = &(apdopts->parameters[stmt->current_exec_param]);
-	ipdopts = SC_get_IPD(stmt);
-	current_iparam = &(ipdopts->parameters[stmt->current_exec_param]);
+	current_param = &(apdopts->parameters[estmt->current_exec_param]);
+	ipdopts = SC_get_IPDF(estmt);
+	current_iparam = &(ipdopts->parameters[estmt->current_exec_param]);
+	pdata = SC_get_PDTI(estmt);
+	current_pdata = &(pdata->pdata[estmt->current_exec_param]);
 	ctype = current_param->CType;
 
-	conn = SC_get_conn(stmt);
+	conn = SC_get_conn(estmt);
 	if (ctype == SQL_C_DEFAULT)
 		ctype = sqltype_to_default_ctype(conn, current_iparam->SQLType);
 	if (SQL_NTS == cbValue)
 	{
-#ifdef	UNICODE_SUPPORT
 		if (SQL_C_WCHAR == ctype)
 		{
-			putlen = 2 * ucs2strlen((SQLWCHAR *) rgbValue);
+			putlen = WCLEN * ucs2strlen((SQLWCHAR *) rgbValue);
 			lenset = TRUE;
 		}
 		else
-#endif /* UNICODE_SUPPORT */
 		if (SQL_C_CHAR == ctype)
 		{
 			putlen = strlen(rgbValue);
@@ -973,11 +1004,7 @@ PGAPI_PutData(
 		if (cbValue < 0)
 			putlen = cbValue;
 		else
-#ifdef	UNICODE_SUPPORT
 		if (ctype == SQL_C_CHAR || ctype == SQL_C_BINARY || ctype == SQL_C_WCHAR)
-#else
-		if (ctype == SQL_C_CHAR || ctype == SQL_C_BINARY)
-#endif /* UNICODE_SUPPORT */
 			putlen = cbValue;
 		else
 			putlen = ctype_length(ctype);
@@ -994,21 +1021,21 @@ PGAPI_PutData(
 		}
 	}
 
-	if (!stmt->put_data)
+	if (!estmt->put_data)
 	{							/* first call */
 		mylog("PGAPI_PutData: (1) cbValue = %d\n", cbValue);
 
-		stmt->put_data = TRUE;
+		estmt->put_data = TRUE;
 
-		current_param->EXEC_used = (SDWORD *) malloc(sizeof(SDWORD));
-		if (!current_param->EXEC_used)
+		current_pdata->EXEC_used = malloc(sizeof(SDWORD));
+		if (!current_pdata->EXEC_used)
 		{
 			SC_set_error(stmt, STMT_NO_MEMORY_ERROR, "Out of memory in PGAPI_PutData (1)");
 			SC_log_error(func, "", stmt);
 			return SQL_ERROR;
 		}
 
-		*current_param->EXEC_used = putlen;
+		*current_pdata->EXEC_used = putlen;
 
 		if (cbValue == SQL_NULL_DATA)
 			return SQL_SUCCESS;
@@ -1018,9 +1045,9 @@ PGAPI_PutData(
 		if (current_iparam->PGType == conn->lobj_type)
 		{
 			/* begin transaction if needed */
-			if (!CC_is_in_trans(stmt->hdbc))
+			if (!CC_is_in_trans(conn))
 			{
-				if (!CC_begin(stmt->hdbc))
+				if (!CC_begin(conn))
 				{
 					SC_set_error(stmt, STMT_EXEC_ERROR, "Could not begin (in-line) a transaction");
 					SC_log_error(func, "", stmt);
@@ -1029,8 +1056,8 @@ PGAPI_PutData(
 			}
 
 			/* store the oid */
-			current_param->lobj_oid = lo_creat(stmt->hdbc, INV_READ | INV_WRITE);
-			if (current_param->lobj_oid == 0)
+			current_pdata->lobj_oid = lo_creat(conn, INV_READ | INV_WRITE);
+			if (current_pdata->lobj_oid == 0)
 			{
 				SC_set_error(stmt, STMT_EXEC_ERROR, "Couldnt create large object.");
 				SC_log_error(func, "", stmt);
@@ -1044,28 +1071,28 @@ PGAPI_PutData(
 			/***current_param->EXEC_buffer = (char *) &current_param->lobj_oid;***/
 
 			/* store the fd */
-			stmt->lobj_fd = lo_open(stmt->hdbc, current_param->lobj_oid, INV_WRITE);
-			if (stmt->lobj_fd < 0)
+			estmt->lobj_fd = lo_open(conn, current_pdata->lobj_oid, INV_WRITE);
+			if (estmt->lobj_fd < 0)
 			{
 				SC_set_error(stmt, STMT_EXEC_ERROR, "Couldnt open large object for writing.");
 				SC_log_error(func, "", stmt);
 				return SQL_ERROR;
 			}
 
-			retval = lo_write(stmt->hdbc, stmt->lobj_fd, putbuf, putlen);
+			retval = lo_write(conn, estmt->lobj_fd, putbuf, putlen);
 			mylog("lo_write: cbValue=%d, wrote %d bytes\n", putlen, retval);
 		}
 		else
 		{
-			current_param->EXEC_buffer = malloc(putlen + 1);
-			if (!current_param->EXEC_buffer)
+			current_pdata->EXEC_buffer = malloc(putlen + 1);
+			if (!current_pdata->EXEC_buffer)
 			{
 				SC_set_error(stmt, STMT_NO_MEMORY_ERROR, "Out of memory in PGAPI_PutData (2)");
 				SC_log_error(func, "", stmt);
 				return SQL_ERROR;
 			}
-			memcpy(current_param->EXEC_buffer, putbuf, putlen);
-			current_param->EXEC_buffer[putlen] = '\0';
+			memcpy(current_pdata->EXEC_buffer, putbuf, putlen);
+			current_pdata->EXEC_buffer[putlen] = '\0';
 		}
 	}
 	else
@@ -1077,23 +1104,23 @@ PGAPI_PutData(
 		if (current_iparam->PGType == conn->lobj_type)
 		{
 			/* the large object fd is in EXEC_buffer */
-			retval = lo_write(stmt->hdbc, stmt->lobj_fd, putbuf, putlen);
+			retval = lo_write(conn, estmt->lobj_fd, putbuf, putlen);
 			mylog("lo_write(2): cbValue = %d, wrote %d bytes\n", putlen, retval);
 
-			*current_param->EXEC_used += putlen;
+			*current_pdata->EXEC_used += putlen;
 		}
 		else
 		{
-			buffer = current_param->EXEC_buffer;
-			old_pos = *current_param->EXEC_used;
+			buffer = current_pdata->EXEC_buffer;
+			old_pos = *current_pdata->EXEC_used;
 			if (putlen > 0)
 			{
-				*current_param->EXEC_used += putlen;
+				*current_pdata->EXEC_used += putlen;
 
-				mylog("        cbValue = %d, old_pos = %d, *used = %d\n", putlen, old_pos, *current_param->EXEC_used);
+				mylog("        cbValue = %d, old_pos = %d, *used = %d\n", putlen, old_pos, *current_pdata->EXEC_used);
 
 				/* dont lose the old pointer in case out of memory */
-				buffer = realloc(current_param->EXEC_buffer, *current_param->EXEC_used + 1);
+				buffer = realloc(current_pdata->EXEC_buffer, *current_pdata->EXEC_used + 1);
 				if (!buffer)
 				{
 					SC_set_error(stmt, STMT_NO_MEMORY_ERROR,"Out of memory in PGAPI_PutData (3)");
@@ -1102,10 +1129,10 @@ PGAPI_PutData(
 				}
 
 				memcpy(&buffer[old_pos], putbuf, putlen);
-				buffer[*current_param->EXEC_used] = '\0';
+				buffer[*current_pdata->EXEC_used] = '\0';
 
 				/* reassign buffer incase realloc moved it */
-				current_param->EXEC_buffer = buffer;
+				current_pdata->EXEC_buffer = buffer;
 			}
 			else
 			{
