@@ -8,7 +8,6 @@
 
 #include "psqlodbc.h"
 
-#if (ODBCVER >= 0x0300)
 #include "connection.h"
 #include "pgapifunc.h"
 
@@ -357,28 +356,32 @@ PGAPI_GetInfo30(HDBC hdbc, UWORD fInfoType, PTR rgbInfoValue,
 			return SQL_ERROR;
 	}
 	result = SQL_SUCCESS;
-	mylog("%s: p='%s', len=%d, value=%d, cbMax=%d\n", func, p ? p : "<NULL>", len, value, cbInfoValueMax);
+
 	if (p)
 	{
 		/* char/binary data */
 		len = strlen(p);
 
+                /* Note that at this point we don't know if we've been called just
+                 * to get the length of the output. If it's unicode, then we better
+                 * adjust to bytes now, so we don't return a buffer size that's too
+                 * small.
+                 */
+                if (conn->unicode)
+                    len = len * WCLEN;
+                
 		if (rgbInfoValue)
 		{
-#ifdef	UNICODE_SUPPORT
+
 			if (conn->unicode)
-			{
 				len = utf8_to_ucs2(p, len, (SQLWCHAR *) rgbInfoValue, cbInfoValueMax / 2);
-				len *= 2;
-			}
 			else
-#endif /* UNICODE_SUPPORT */
-			strncpy_null((char *) rgbInfoValue, p, (size_t) cbInfoValueMax);
+				strncpy_null((char *) rgbInfoValue, p, (size_t) cbInfoValueMax);
 
 			if (len >= cbInfoValueMax)
 			{
 				result = SQL_SUCCESS_WITH_INFO;
-				CC_set_error(conn, CONN_TRUNCATED, "The buffer was too small for tthe InfoValue.");
+				CC_set_error(conn, CONN_TRUNCATED, "The buffer was too small for the InfoValue.");
 			}
 		}
 	}
@@ -396,6 +399,9 @@ PGAPI_GetInfo30(HDBC hdbc, UWORD fInfoType, PTR rgbInfoValue,
 
 	if (pcbInfoValue)
 		*pcbInfoValue = len;
+
+	mylog("%s: p='%s', len=%d, value=%d, cbMax=%d\n", func, p ? p : "<NULL>", len, value, cbInfoValueMax);
+	
 	return result;
 }
-#endif /* ODBCVER >= 0x0300 */
+
