@@ -110,12 +110,12 @@ logs_on_off(int cnopen, int mylog_onoff, int qlog_onoff)
 }
 
 #ifdef MY_LOG
+static FILE *LOGFP = NULL;
 void
 mylog(char *fmt,...)
 {
 	va_list		args;
 	char		filebuf[80];
-	static FILE *LOGFP = NULL;
 
 	ENTER_MYLOG_CS;
 	if (mylog_on)
@@ -136,8 +136,8 @@ mylog(char *fmt,...)
 #endif /* WIN32 */
 #endif /* WIN_MULTITHREAD_SUPPORT */
 #if defined(POSIX_MULTITHREAD_SUPPORT)
-        if (LOGFP)
-            fprintf(LOGFP, "[%d]", pthread_self());
+		if (LOGFP)
+			fprintf(LOGFP, "[%d]", pthread_self());
 #endif /* POSIX_MULTITHREAD_SUPPORT */
 		if (LOGFP)
 			vfprintf(LOGFP, fmt, args);
@@ -259,14 +259,15 @@ strncpy_null(char *dst, const char *src, int len)
 /*------
  *	Create a null terminated string (handling the SQL_NTS thing):
  *		1. If buf is supplied, place the string in there
- *		   (assumes enough space) and return buf.
+ *		   (at most bufsize-1 bytes) and return buf.
  *		2. If buf is not supplied, malloc space and return this string
+ *		   (buflen is ignored in this case)
  *------
  */
 char *
-make_string(const char *s, int len, char *buf)
+make_string(const char *s, int len, char *buf, size_t bufsize)
 {
-	int			length;
+	unsigned int	length;
 	char	   *str;
 
 	if (s && (len > 0 || (len == SQL_NTS && strlen(s) > 0)))
@@ -275,6 +276,8 @@ make_string(const char *s, int len, char *buf)
 
 		if (buf)
 		{
+			if (length >= bufsize)
+				length = bufsize - 1;
 			strncpy_null(buf, s, length + 1);
 			return buf;
 		}
