@@ -11,6 +11,7 @@
 #include "multibyte.h"
 #include "connection.h"
 #include "pgapifunc.h"
+#include "qresult.h"
 #include <string.h>
 #include <ctype.h>
 #include <stdio.h>
@@ -75,7 +76,7 @@ pg_ismb(int characterset_code)
 #endif
 
 int
-pg_CS_code(const unsigned char *characterset_string)
+pg_CS_code(const UCHAR *characterset_string)
 {
 	int i = 0, c = -1;
   	unsigned len = 0;
@@ -96,7 +97,7 @@ pg_CS_code(const unsigned char *characterset_string)
 	return (c);
 }
 
-unsigned char *
+UCHAR *
 pg_CS_name(int characterset_code)
 {
 	int i;
@@ -273,30 +274,30 @@ pg_CS_stat(int stat,unsigned int character,int characterset_code)
 }
 
 
-unsigned char *
-pg_mbschr(int csc, const unsigned char *string, unsigned int character)
+UCHAR *
+pg_mbschr(int csc, const UCHAR *string, unsigned int character)
 {
 	int			mb_st = 0;
-	const unsigned char *s, *rs = NULL;
+	const UCHAR *s, *rs = NULL;
 
 	for(s = string; *s ; s++) 
 	{
-		mb_st = pg_CS_stat(mb_st, (unsigned char) *s, csc);
+		mb_st = pg_CS_stat(mb_st, (UCHAR) *s, csc);
 		if (mb_st == 0 && (*s == character))
 		{
 			rs = s;
 			break;
 		}
 	}
-	return ((unsigned char *) rs);
+	return ((UCHAR *) rs);
 }
 
 int
-pg_mbslen(int csc, const unsigned char *string)
+pg_mbslen(int csc, const UCHAR *string)
 {
-	unsigned char *s;
+	UCHAR *s;
 	int len, cs_stat;
-	for (len = 0, cs_stat = 0, s = (unsigned char *) string; *s != 0; s++)
+	for (len = 0, cs_stat = 0, s = (UCHAR *) string; *s != 0; s++)
 	{
 		cs_stat = pg_CS_stat(cs_stat,(unsigned int) *s, csc);
 		if (cs_stat < 2)
@@ -305,8 +306,8 @@ pg_mbslen(int csc, const unsigned char *string)
 	return len;
 }
 
-unsigned char *
-pg_mbsinc(int csc, const unsigned char *current )
+UCHAR *
+pg_mbsinc(int csc, const UCHAR *current )
 {
 	int mb_stat = 0;
 	if (*current != 0)
@@ -314,7 +315,7 @@ pg_mbsinc(int csc, const unsigned char *current )
 		mb_stat = (int) pg_CS_stat(mb_stat, *current, csc);
 		if (mb_stat == 0)
 			mb_stat = 1;
-		return ((unsigned char *) current + mb_stat);
+		return ((UCHAR *) current + mb_stat);
 	}
 	else
 		return NULL;
@@ -348,7 +349,7 @@ CC_lookup_cs_old(ConnectionClass *self)
 	if ((result != SQL_SUCCESS) && (result != SQL_SUCCESS_WITH_INFO))
 		return encstr;
 
-	result = PGAPI_ExecDirect(hstmt, "Show Client_Encoding", SQL_NTS);
+	result = PGAPI_ExecDirect(hstmt, "Show Client_Encoding", SQL_NTS, 0);
 	if (result == SQL_SUCCESS_WITH_INFO)
 	{
 		char sqlState[8], errormsg[128], enc[32];
@@ -375,50 +376,7 @@ CC_lookup_characterset(ConnectionClass *self)
 		encstr = CC_lookup_cs_new(self);
 	if (self->client_encoding)
 		free(self->client_encoding);
-#ifndef	UNICODE_SUPPORT
-#ifdef	WIN32
-	else
-	{
-		const char *wenc = NULL;
-		switch (GetACP())
-		{
-			case 932:
-				wenc = "SJIS";
-				break;
-			case 936:
-				if (!encstr || PG_VERSION_GT(self, 7.2))
-					wenc = "GBK";
-				break;
-			case 949:
-				if (!encstr ||
-				  (PG_VERSION_GT(self, 7.2) && stricmp(encstr, "EUC_KR")))  
-					wenc = "UHC";
-				break;
-			case 950:
-				wenc = "BIG5";
-				break;
-		}
-		if (wenc && (!encstr || stricmp(encstr, wenc)))
-		{
-			QResultClass	*res;
-			char		query[64];
-			int		errnum = CC_get_errornumber(self);
 
-			sprintf(query, "set client_encoding to '%s'", wenc);
-			res = CC_send_query(self, query, NULL, CLEAR_RESULT_ON_ABORT);
-			CC_set_errornumber(self, errnum);
-			if (res)
-			{
-				self->client_encoding = strdup(wenc);
-				self->ccsc = pg_CS_code(self->client_encoding);
-				QR_Destructor(res);
-				free(encstr);
-				return;
-			}
-		}
-	}
-#endif /* WIN32 */
-#endif /* UNICODE_SUPPORT */
 	if (encstr)
 	{
 		self->client_encoding = encstr;
