@@ -36,7 +36,6 @@
 #include "statement.h"
 #include "qresult.h"
 
-#if (ODBCVER < 0x0300)
 RETCODE		SQL_API
 SQLAllocConnect(HENV EnvironmentHandle,
 				HDBC FAR * ConnectionHandle)
@@ -75,7 +74,6 @@ SQLAllocStmt(HDBC ConnectionHandle,
 	LEAVE_CONN_CS(conn);
 	return ret;
 }
-#endif /* ODBCVER */
 
 RETCODE		SQL_API
 SQLBindCol(HSTMT StatementHandle,
@@ -110,17 +108,22 @@ SQLColumns(HSTMT StatementHandle,
 		   SQLCHAR *TableName, SQLSMALLINT NameLength3,
 		   SQLCHAR *ColumnName, SQLSMALLINT NameLength4)
 {
+	CSTR func = "SQLColumns";
 	RETCODE	ret;
 	StatementClass *stmt = (StatementClass *) StatementHandle;
 	SQLCHAR	*ctName = CatalogName, *scName = SchemaName,
 		*tbName = TableName, *clName = ColumnName;
+	UWORD	flag = PODBC_SEARCH_PUBLIC_SCHEMA;
 
-	mylog("[SQLColumns]");
+	mylog("[%s]", func);
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
-	ret = PGAPI_Columns(StatementHandle, ctName, NameLength1,
+	if (SC_opencheck(stmt, func))
+		ret = SQL_ERROR;
+	else
+		ret = PGAPI_Columns(StatementHandle, ctName, NameLength1,
 				scName, NameLength2, tbName, NameLength3,
-				clName, NameLength4, 0);
+				clName, NameLength4, flag);
 	if (SQL_SUCCESS == ret && 0 == QR_get_num_total_tuples(SC_get_Result(stmt))) 
 	{
 		BOOL	ifallupper = TRUE, reexec = FALSE;
@@ -153,7 +156,7 @@ SQLColumns(HSTMT StatementHandle,
 		{
 			ret = PGAPI_Columns(StatementHandle, ctName, NameLength1,
 				scName, NameLength2, tbName, NameLength3,
-				clName, NameLength4, 0);
+				clName, NameLength4, flag);
 			if (newCt)
 				free(newCt);
 			if (newSc)
@@ -280,39 +283,21 @@ SQLDisconnect(HDBC ConnectionHandle)
 	return ret;
 }
 
-#if (ODBCVER < 0x0300)
-RETCODE		SQL_API
-SQLError(HENV EnvironmentHandle,
-		 HDBC ConnectionHandle, HSTMT StatementHandle,
-		 SQLCHAR *Sqlstate, SQLINTEGER *NativeError,
-		 SQLCHAR *MessageText, SQLSMALLINT BufferLength,
-		 SQLSMALLINT *TextLength)
-{
-	RETCODE	ret;
-
-	mylog("[SQLError]");
-	if (NULL != EnvironmentHandle)
-		ENTER_ENV_CS((EnvironmentClass *) EnvironmentHandle);
-	ret = PGAPI_Error(EnvironmentHandle, ConnectionHandle, StatementHandle,
-		   Sqlstate, NativeError, MessageText, BufferLength,
-		TextLength);
-	if (NULL != EnvironmentHandle)
-		LEAVE_ENV_CS((EnvironmentClass *) EnvironmentHandle);
-	return ret;
-}
-#endif /* ODBCVER */
-
 RETCODE		SQL_API
 SQLExecDirect(HSTMT StatementHandle,
 			  SQLCHAR *StatementText, SQLINTEGER TextLength)
 {
+	CSTR func = "SQLExecDirect";
 	RETCODE	ret;
 	StatementClass *stmt = (StatementClass *) StatementHandle;
 
-	mylog("[SQLExecDirect]");
+	mylog("[%s]", func);
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
-	ret = PGAPI_ExecDirect(StatementHandle, StatementText, TextLength);
+	if (SC_opencheck(stmt, func))
+		ret = SQL_ERROR;
+	else
+		ret = PGAPI_ExecDirect(StatementHandle, StatementText, TextLength, 0);
 	LEAVE_STMT_CS(stmt);
 	return ret;
 }
@@ -320,13 +305,17 @@ SQLExecDirect(HSTMT StatementHandle,
 RETCODE		SQL_API
 SQLExecute(HSTMT StatementHandle)
 {
+	CSTR func = "SQLExecute";
 	RETCODE	ret;
 	StatementClass *stmt = (StatementClass *) StatementHandle;
 
-	mylog("[SQLExecute]");
+	mylog("[%s]", func);
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
-	ret = PGAPI_Execute(StatementHandle);
+	if (SC_opencheck(stmt, func))
+		ret = SQL_ERROR;
+	else
+		ret = PGAPI_Execute(StatementHandle, 0);
 	LEAVE_STMT_CS(stmt);
 	return ret;
 }
@@ -340,13 +329,12 @@ SQLFetch(HSTMT StatementHandle)
 
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
-#if (ODBCVER >= 0x0300)
 	if (SC_get_conn(stmt)->driver_version >= 0x0300)
 	{
-		IRDFields	*irdopts = SC_get_IRD(stmt);
-		ARDFields	*ardopts = SC_get_ARD(stmt);
+		IRDFields	*irdopts = SC_get_IRDF(stmt);
+		ARDFields	*ardopts = SC_get_ARDF(stmt);
 		SQLUSMALLINT *rowStatusArray = irdopts->rowStatusArray;
-		SQLINTEGER *pcRow = irdopts->rowsFetched;
+		SQLINTEGER *pcRow = (SQLINTEGER *) irdopts->rowsFetched;
 
 		mylog("[[%s]]", func);
 		ret = PGAPI_ExtendedFetch(StatementHandle, SQL_FETCH_NEXT, 0,
@@ -354,7 +342,6 @@ SQLFetch(HSTMT StatementHandle)
 		stmt->transition_status = 6;
 	}
 	else
-#endif
 	{
 		mylog("[%s]", func);
 		ret = PGAPI_Fetch(StatementHandle);
@@ -363,28 +350,6 @@ SQLFetch(HSTMT StatementHandle)
 	return ret;
 }
 
-#if (ODBCVER < 0x0300) 
-RETCODE		SQL_API
-SQLFreeConnect(HDBC ConnectionHandle)
-{
-	RETCODE	ret;
-
-	mylog("[SQLFreeConnect]");
-	ret = PGAPI_FreeConnect(ConnectionHandle);
-	return ret;
-}
-
-RETCODE		SQL_API
-SQLFreeEnv(HENV EnvironmentHandle)
-{
-	RETCODE	ret;
-
-	mylog("[SQLFreeEnv]");
-	ret = PGAPI_FreeEnv(EnvironmentHandle);
-	return ret;
-}
-#endif /* ODBCVER */ 
-
 RETCODE		SQL_API
 SQLFreeStmt(HSTMT StatementHandle,
 			SQLUSMALLINT Option)
@@ -396,23 +361,6 @@ SQLFreeStmt(HSTMT StatementHandle,
 	return ret;
 }
 
-#if (ODBCVER < 0x0300)
-RETCODE		SQL_API
-SQLGetConnectOption(HDBC ConnectionHandle,
-					SQLUSMALLINT Option, PTR Value)
-{
-	RETCODE	ret;
-	ConnectionClass *conn = (ConnectionClass *) ConnectionHandle;
-
-	mylog("[SQLGetConnectOption]");
-	ENTER_CONN_CS(conn);
-	CC_clear_error(conn);
-	ret = PGAPI_GetConnectOption(ConnectionHandle, Option, Value);
-	LEAVE_CONN_CS(conn);
-	return ret;
-}
-#endif /* ODBCVER */
-
 RETCODE		SQL_API
 SQLGetCursorName(HSTMT StatementHandle,
 				 SQLCHAR *CursorName, SQLSMALLINT BufferLength,
@@ -458,11 +406,10 @@ SQLGetFunctions(HDBC ConnectionHandle,
 	mylog("[SQLGetFunctions]");
 	ENTER_CONN_CS(conn);
 	CC_clear_error(conn);
-#if (ODBCVER >= 0x0300)
+
 	if (FunctionId == SQL_API_ODBC3_ALL_FUNCTIONS)
 		ret = PGAPI_GetFunctions30(ConnectionHandle, FunctionId, Supported);
 	else
-#endif
 	{
 		ret = PGAPI_GetFunctions(ConnectionHandle, FunctionId, Supported);
 	}
@@ -479,7 +426,7 @@ SQLGetInfo(HDBC ConnectionHandle,
 
 	ENTER_CONN_CS(conn);
 	CC_clear_error(conn);
-#if (ODBCVER >= 0x0300)
+
 	mylog("[SQLGetInfo(30)]");
 	if ((ret = PGAPI_GetInfo(ConnectionHandle, InfoType, InfoValue,
 							 BufferLength, StringLength)) == SQL_ERROR)
@@ -493,44 +440,26 @@ SQLGetInfo(HDBC ConnectionHandle,
 	}
 	if (SQL_ERROR == ret)
 		CC_log_error("SQLGetInfo30", "", conn);
-#else
-	mylog("[SQLGetInfo]");
-	if (ret = PGAPI_GetInfo(ConnectionHandle, InfoType, InfoValue,
-			BufferLength, StringLength), SQL_ERROR == ret)
-		CC_log_error("PGAPI_GetInfo", "", conn);
-#endif
+
 	LEAVE_CONN_CS(conn);
 	return ret;
 }
 
-#if (ODBCVER < 0x0300)
-RETCODE		SQL_API
-SQLGetStmtOption(HSTMT StatementHandle,
-				 SQLUSMALLINT Option, PTR Value)
-{
-	RETCODE	ret;
-	StatementClass *stmt = (StatementClass *) StatementHandle;
-
-	mylog("[SQLGetStmtOption]");
-	ENTER_STMT_CS(stmt);
-	SC_clear_error(stmt);
-	ret = PGAPI_GetStmtOption(StatementHandle, Option, Value);
-	LEAVE_STMT_CS(stmt);
-	return ret;
-}
-#endif /* ODBCVER */
-
 RETCODE		SQL_API
 SQLGetTypeInfo(HSTMT StatementHandle,
 			   SQLSMALLINT DataType)
 {
+	CSTR func = "SQLGetTypeInfo";
 	RETCODE	ret;
 	StatementClass *stmt = (StatementClass *) StatementHandle;
 
-	mylog("[SQLGetTypeInfo]");
+	mylog("[%s]", func);
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
-	ret = PGAPI_GetTypeInfo(StatementHandle, DataType);
+	if (SC_opencheck(stmt, func))
+		ret = SQL_ERROR;
+	else
+		ret = PGAPI_GetTypeInfo(StatementHandle, DataType);
 	LEAVE_STMT_CS(stmt);
 	return ret;
 }
@@ -610,23 +539,6 @@ SQLRowCount(HSTMT StatementHandle,
 	return ret;
 }
 
-#if (ODBCVER < 0x0300)
-RETCODE		SQL_API
-SQLSetConnectOption(HDBC ConnectionHandle,
-					SQLUSMALLINT Option, SQLUINTEGER Value)
-{
-	RETCODE	ret;
-	ConnectionClass *conn = (ConnectionClass *) ConnectionHandle;
-
-	mylog("[SQLSetConnectionOption]");
-	ENTER_CONN_CS(conn);
-	CC_clear_error(conn);
-	ret = PGAPI_SetConnectOption(ConnectionHandle, Option, Value);
-	LEAVE_CONN_CS(conn);
-	return ret;
-}
-#endif /* ODBCVER */
-
 RETCODE		SQL_API
 SQLSetCursorName(HSTMT StatementHandle,
 				 SQLCHAR *CursorName, SQLSMALLINT NameLength)
@@ -660,23 +572,6 @@ SQLSetParam(HSTMT StatementHandle,
 	return SQL_ERROR;
 }
 
-#if (ODBCVER < 0x0300)
-RETCODE		SQL_API
-SQLSetStmtOption(HSTMT StatementHandle,
-				 SQLUSMALLINT Option, SQLUINTEGER Value)
-{
-	RETCODE	ret;
-	StatementClass *stmt = (StatementClass *) StatementHandle;
-
-	mylog("[SQLSetStmtOption]");
-	ENTER_STMT_CS(stmt);
-	SC_clear_error(stmt);
-	ret = PGAPI_SetStmtOption(StatementHandle, Option, Value);
-	LEAVE_STMT_CS(stmt);
-	return ret;
-}
-#endif /* ODBCVER */
-
 RETCODE		SQL_API
 SQLSpecialColumns(HSTMT StatementHandle,
 				  SQLUSMALLINT IdentifierType, SQLCHAR *CatalogName,
@@ -685,14 +580,18 @@ SQLSpecialColumns(HSTMT StatementHandle,
 				  SQLSMALLINT NameLength3, SQLUSMALLINT Scope,
 				  SQLUSMALLINT Nullable)
 {
+	CSTR func = "SQLSpecialColumns";
 	RETCODE	ret;
 	StatementClass *stmt = (StatementClass *) StatementHandle;
 	SQLCHAR *ctName = CatalogName, *scName = SchemaName, *tbName = TableName;
 
-	mylog("[SQLSpecialColumns]");
+	mylog("[%s]", func);
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
-	ret = PGAPI_SpecialColumns(StatementHandle, IdentifierType, ctName,
+	if (SC_opencheck(stmt, func))
+		ret = SQL_ERROR;
+	else
+		ret = PGAPI_SpecialColumns(StatementHandle, IdentifierType, ctName,
 			NameLength1, scName, NameLength2, tbName, NameLength3,
 							Scope, Nullable);
 	if (SQL_SUCCESS == ret && 0 == QR_get_num_total_tuples(SC_get_Result(stmt))) 
@@ -742,14 +641,18 @@ SQLStatistics(HSTMT StatementHandle,
 			  SQLCHAR *TableName, SQLSMALLINT NameLength3,
 			  SQLUSMALLINT Unique, SQLUSMALLINT Reserved)
 {
+	CSTR func = "SQLStatistics";
 	RETCODE	ret;
 	StatementClass *stmt = (StatementClass *) StatementHandle;
 	SQLCHAR *ctName = CatalogName, *scName = SchemaName, *tbName = TableName;
 
-	mylog("[SQLStatistics]");
+	mylog("[%s]", func);
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
-	ret = PGAPI_Statistics(StatementHandle, ctName, NameLength1,
+	if (SC_opencheck(stmt, func))
+		ret = SQL_ERROR;
+	else
+		ret = PGAPI_Statistics(StatementHandle, ctName, NameLength1,
 				 scName, NameLength2, tbName, NameLength3,
 				 Unique, Reserved);
 	if (SQL_SUCCESS == ret && 0 == QR_get_num_total_tuples(SC_get_Result(stmt))) 
@@ -799,14 +702,18 @@ SQLTables(HSTMT StatementHandle,
 		  SQLCHAR *TableName, SQLSMALLINT NameLength3,
 		  SQLCHAR *TableType, SQLSMALLINT NameLength4)
 {
+	CSTR func = "SQLTables";
 	RETCODE	ret;
 	StatementClass *stmt = (StatementClass *) StatementHandle;
 	SQLCHAR *ctName = CatalogName, *scName = SchemaName, *tbName = TableName;
 
-	mylog("[SQLTables]");
+	mylog("[%s]", func);
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
-	ret = PGAPI_Tables(StatementHandle, ctName, NameLength1,
+	if (SC_opencheck(stmt, func))
+		ret = SQL_ERROR;
+	else
+		ret = PGAPI_Tables(StatementHandle, ctName, NameLength1,
 				scName, NameLength2, tbName, NameLength3,
 						TableType, NameLength4);
 	if (SQL_SUCCESS == ret && 0 == QR_get_num_total_tuples(SC_get_Result(stmt))) 
@@ -849,49 +756,6 @@ SQLTables(HSTMT StatementHandle,
 	return ret;
 }
 
-#if (ODBCVER < 0x0300)
-RETCODE		SQL_API
-SQLTransact(HENV EnvironmentHandle,
-			HDBC ConnectionHandle, SQLUSMALLINT CompletionType)
-{
-	RETCODE	ret;
-
-	mylog("[SQLTransact]");
-	if (NULL != EnvironmentHandle)
-		ENTER_ENV_CS((EnvironmentClass *) EnvironmentHandle);
-	else
-		ENTER_CONN_CS((ConnectionClass *) ConnectionHandle);
-	ret = PGAPI_Transact(EnvironmentHandle, ConnectionHandle, CompletionType);
-	if (NULL != EnvironmentHandle)
-		LEAVE_ENV_CS((EnvironmentClass *) EnvironmentHandle);
-	else
-		LEAVE_CONN_CS((ConnectionClass *) ConnectionHandle);
-	return ret;
-}
-
-RETCODE		SQL_API
-SQLColAttributes(
-				 HSTMT hstmt,
-				 SQLUSMALLINT icol,
-				 SQLUSMALLINT fDescType,
-				 PTR rgbDesc,
-				 SQLSMALLINT cbDescMax,
-				 SQLSMALLINT *pcbDesc,
-				 SQLINTEGER *pfDesc)
-{
-	RETCODE	ret;
-	StatementClass *stmt = (StatementClass *) hstmt;
-
-	mylog("[SQLColAttributes]");
-	ENTER_STMT_CS(stmt);
-	SC_clear_error(stmt);
-	ret = PGAPI_ColAttributes(hstmt, icol, fDescType, rgbDesc,
-							   cbDescMax, pcbDesc, pfDesc);
-	LEAVE_STMT_CS(stmt);
-	return ret;
-}
-#endif /* ODBCVER */
-
 RETCODE		SQL_API
 SQLColumnPrivileges(
 					HSTMT hstmt,
@@ -904,15 +768,19 @@ SQLColumnPrivileges(
 					SQLCHAR *szColumnName,
 					SQLSMALLINT cbColumnName)
 {
+	CSTR func = "SQLColumnPrivileges";
 	RETCODE	ret;
 	StatementClass *stmt = (StatementClass *) hstmt;
 	SQLCHAR	*ctName = szCatalogName, *scName = szSchemaName,
 		*tbName = szTableName, *clName = szColumnName;
 
-	mylog("[SQLColumnPrivileges]");
+	mylog("[%s]", func);
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
-	ret = PGAPI_ColumnPrivileges(hstmt, ctName, cbCatalogName,
+	if (SC_opencheck(stmt, func))
+		ret = SQL_ERROR;
+	else
+		ret = PGAPI_ColumnPrivileges(hstmt, ctName, cbCatalogName,
 				scName, cbSchemaName, tbName, cbTableName,
 						clName, cbColumnName);
 	if (SQL_SUCCESS == ret && 0 == QR_get_num_total_tuples(SC_get_Result(stmt))) 
@@ -997,7 +865,7 @@ SQLExtendedFetch(
 	mylog("[SQLExtendedFetch]");
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
-	ret = PGAPI_ExtendedFetch(hstmt, fFetchType, irow, pcrow, rgfRowStatus, 0, SC_get_ARD(stmt)->size_of_rowset_odbc2);
+	ret = PGAPI_ExtendedFetch(hstmt, fFetchType, irow, pcrow, rgfRowStatus, 0, SC_get_ARDF(stmt)->size_of_rowset_odbc2);
 	stmt->transition_status = 7;
 	LEAVE_STMT_CS(stmt);
 	return ret;
@@ -1019,16 +887,20 @@ SQLForeignKeys(
 			   SQLCHAR *szFkTableName,
 			   SQLSMALLINT cbFkTableName)
 {
+	CSTR func = "SQLForeignKeys";
 	RETCODE	ret;
 	StatementClass *stmt = (StatementClass *) hstmt;
 	SQLCHAR *pkctName = szPkCatalogName, *pkscName = szPkSchemaName,
 		*pktbName = szPkTableName, *fkctName = szFkCatalogName,
 		*fkscName = szFkSchemaName, *fktbName = szFkTableName;
 
-	mylog("[SQLForeignKeys]");
+	mylog("[%s]", func);
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
-	ret = PGAPI_ForeignKeys(hstmt, pkctName, cbPkCatalogName,
+	if (SC_opencheck(stmt, func))
+		ret = SQL_ERROR;
+	else
+		ret = PGAPI_ForeignKeys(hstmt, pkctName, cbPkCatalogName,
 			pkscName, cbPkSchemaName, pktbName, cbPkTableName,
 			fkctName, cbFkCatalogName, fkscName, cbFkSchemaName,
 			fktbName, cbFkTableName);
@@ -1173,15 +1045,19 @@ SQLPrimaryKeys(
 			   SQLCHAR *szTableName,
 			   SQLSMALLINT cbTableName)
 {
+	CSTR func = "SQLPrimaryKeys";
 	RETCODE	ret;
 	StatementClass *stmt = (StatementClass *) hstmt;
 	SQLCHAR	*ctName = szCatalogName, *scName = szSchemaName,
 		*tbName = szTableName;
 
-	mylog("[SQLPrimaryKeys]");
+	mylog("[%s]", func);
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
-	ret = PGAPI_PrimaryKeys(hstmt, ctName, cbCatalogName,
+	if (SC_opencheck(stmt, func))
+		ret = SQL_ERROR;
+	else
+		ret = PGAPI_PrimaryKeys(hstmt, ctName, cbCatalogName,
 			scName, cbSchemaName, tbName, cbTableName);
 	if (SQL_SUCCESS == ret && 0 == QR_get_num_total_tuples(SC_get_Result(stmt))) 
 	{
@@ -1234,15 +1110,19 @@ SQLProcedureColumns(
 					SQLCHAR *szColumnName,
 					SQLSMALLINT cbColumnName)
 {
+	CSTR func = "SQLProcedureColumns";
 	RETCODE	ret;
 	StatementClass *stmt = (StatementClass *) hstmt;
 	SQLCHAR	*ctName = szCatalogName, *scName = szSchemaName,
 		*prName = szProcName, *clName = szColumnName;
 
-	mylog("[SQLProcedureColumns]");
+	mylog("[%s]", func);
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
-	ret = PGAPI_ProcedureColumns(hstmt, ctName, cbCatalogName,
+	if (SC_opencheck(stmt, func))
+		ret = SQL_ERROR;
+	else
+		ret = PGAPI_ProcedureColumns(hstmt, ctName, cbCatalogName,
 				scName, cbSchemaName, prName, cbProcName,
 					clName, cbColumnName);
 	if (SQL_SUCCESS == ret && 0 == QR_get_num_total_tuples(SC_get_Result(stmt))) 
@@ -1302,15 +1182,19 @@ SQLProcedures(
 			  SQLCHAR *szProcName,
 			  SQLSMALLINT cbProcName)
 {
+	CSTR func = "SQLProcedures";
 	RETCODE	ret;
 	StatementClass *stmt = (StatementClass *) hstmt;
 	SQLCHAR	*ctName = szCatalogName, *scName = szSchemaName,
 		*prName = szProcName;
 
-	mylog("[SQLProcedures]");
+	mylog("[%s]", func);
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
-	ret = PGAPI_Procedures(hstmt, ctName, cbCatalogName,
+	if (SC_opencheck(stmt, func))
+		ret = SQL_ERROR;
+	else
+		ret = PGAPI_Procedures(hstmt, ctName, cbCatalogName,
 					 scName, cbSchemaName, prName, cbProcName);
 	if (SQL_SUCCESS == ret && 0 == QR_get_num_total_tuples(SC_get_Result(stmt))) 
 	{
@@ -1379,15 +1263,19 @@ SQLTablePrivileges(
 				   SQLCHAR *szTableName,
 				   SQLSMALLINT cbTableName)
 {
+	CSTR func = "SQLTablePrivileges";
 	RETCODE	ret;
 	StatementClass *stmt = (StatementClass *) hstmt;
 	SQLCHAR	*ctName = szCatalogName, *scName = szSchemaName,
 		*tbName = szTableName;
 
-	mylog("[SQLTablePrivileges]");
+	mylog("[%s]", func);
 	ENTER_STMT_CS(stmt);
 	SC_clear_error(stmt);
-	ret = PGAPI_TablePrivileges(hstmt, ctName, cbCatalogName,
+	if (SC_opencheck(stmt, func))
+		ret = SQL_ERROR;
+	else
+		ret = PGAPI_TablePrivileges(hstmt, ctName, cbCatalogName,
 			scName, cbSchemaName, tbName, cbTableName, 0);
 	if (SQL_SUCCESS == ret && 0 == QR_get_num_total_tuples(SC_get_Result(stmt))) 
 	{
