@@ -55,7 +55,7 @@ set_statement_option(ConnectionClass *conn,
 			if (conn)
 				conn->ardOptions.bind_size = vParam;
 			if (stmt)
-				SC_get_ARD(stmt)->bind_size = vParam;
+				SC_get_ARDF(stmt)->bind_size = vParam;
 			break;
 
 		case SQL_CONCURRENCY:
@@ -215,7 +215,7 @@ set_statement_option(ConnectionClass *conn,
 			 */
 
 			if (stmt && stmt->save_rowset_size <= 0 && stmt->last_fetch_count > 0)
-				stmt->save_rowset_size = SC_get_ARD(stmt)->size_of_rowset_odbc2;
+				stmt->save_rowset_size = SC_get_ARDF(stmt)->size_of_rowset_odbc2;
 
 			if (vParam < 1)
 			{
@@ -226,7 +226,7 @@ set_statement_option(ConnectionClass *conn,
 			if (conn)
 				conn->ardOptions.size_of_rowset_odbc2 = vParam;
 			if (stmt)
-				SC_get_ARD(stmt)->size_of_rowset_odbc2 = vParam;
+				SC_get_ARDF(stmt)->size_of_rowset_odbc2 = vParam;
 			break;
 
 		case SQL_SIMULATE_CURSOR:		/* NOT SUPPORTED */
@@ -249,8 +249,9 @@ set_statement_option(ConnectionClass *conn,
 				conn->stmtOptions.use_bookmarks = vParam;
 			break;
 
-		case 1227:
-		case 1228:
+		case 1204: /* SQL_COPT_SS_PRESERVE_CURSORS ? */
+		case 1227: /* SQL_SOPT_SS_HIDDEN_COLUMNS ? */
+		case 1228: /* SQL_SOPT_SS_NOBROWSETABLE ? */
 			if (stmt)
 			{
 				SC_set_error(stmt, STMT_OPTION_NOT_FOR_THE_DRIVER, "The option may be for MS SQL Server(Set)");
@@ -476,7 +477,6 @@ PGAPI_SetConnectOption(
 				if (fOption == 30002 && vParam)
 				{
 					int	cmp;
-#ifdef	UNICODE_SUPPORT
 					char *asPara;
 					if (conn->unicode)
 					{
@@ -485,8 +485,8 @@ PGAPI_SetConnectOption(
 						free(asPara);
 					}
 					else
-#endif /* UNICODE_SUPPORT */
-					cmp = strncmp((char *) vParam, "Microsoft Jet", 13);
+						cmp = strncmp((char *) vParam, "Microsoft Jet", 13);
+					
 					if (0 == cmp)
 					{
 						mylog("Microsoft Jet !!!!\n");
@@ -699,7 +699,7 @@ PGAPI_GetStmtOption(
 			break;
 
 		case SQL_BIND_TYPE:
-			*((SDWORD *) pvParam) = SC_get_ARD(stmt)->bind_size;
+			*((SDWORD *) pvParam) = SC_get_ARDF(stmt)->bind_size;
 			break;
 
 		case SQL_CONCURRENCY:	/* NOT REALLY SUPPORTED */
@@ -739,7 +739,7 @@ PGAPI_GetStmtOption(
 			break;
 
 		case SQL_ROWSET_SIZE:
-			*((SDWORD *) pvParam) = SC_get_ARD(stmt)->size_of_rowset_odbc2;
+			*((SDWORD *) pvParam) = SC_get_ARDF(stmt)->size_of_rowset_odbc2;
 			break;
 
 		case SQL_SIMULATE_CURSOR:		/* NOT SUPPORTED */
