@@ -85,7 +85,7 @@ getNextToken(
 	smax--;
 
 	/* skip leading delimiters */
-	while (isspace((unsigned char) s[i]) || s[i] == ',')
+	while (isspace((UCHAR) s[i]) || s[i] == ',')
 	{
 		/* mylog("skipping '%c'\n", s[i]); */
 		i++;
@@ -114,7 +114,7 @@ getNextToken(
 			token[out++] = s[i++];
 			continue;
 		}
-		if (isspace((unsigned char) s[i]) || s[i] == ',')
+		if (isspace((UCHAR) s[i]) || s[i] == ',')
 			break;
 		/* Handle quoted stuff */
 		if (out == 0 && (s[i] == '\"' || s[i] == '\''))
@@ -157,18 +157,18 @@ getNextToken(
 		}
 
 		/* Check for numeric literals */
-		if (out == 0 && isdigit((unsigned char) s[i]))
+		if (out == 0 && isdigit((UCHAR) s[i]))
 		{
 			if (numeric)
 				*numeric = TRUE;
 			token[out++] = s[i++];
-			while (isalnum((unsigned char) s[i]) || s[i] == '.')
+			while (isalnum((UCHAR) s[i]) || s[i] == '.')
 				token[out++] = s[i++];
 
 			break;
 		}
 
-		if (ispunct((unsigned char) s[i]) && s[i] != '_')
+		if (ispunct((UCHAR) s[i]) && s[i] != '_')
 		{
 			mylog("got ispunct: s[%d] = '%c'\n", i, s[i]);
 
@@ -192,7 +192,7 @@ getNextToken(
 	token[out] = '\0';
 
 	/* find the delimiter  */
-	while (isspace((unsigned char) s[i]))
+	while (isspace((UCHAR) s[i]))
 		i++;
 
 	/* return the most priority delimiter */
@@ -213,7 +213,7 @@ getNextToken(
 	}
 
 	/* skip trailing blanks  */
-	while (isspace((unsigned char) s[i]))
+	while (isspace((UCHAR) s[i]))
 		i++;
 
 	return &s[i];
@@ -245,11 +245,8 @@ getColInfo(COL_INFO *col_info, FIELD_INFO *fi, int k)
 	char	   *str;
 	Int2	reserved_cols;
 
-#if (ODBCVER >= 0x0300)
 	reserved_cols = 18;
-#else
-	reserved_cols = 12;
-#endif /* ODBCVER */
+
 	if (fi->name[0] == '\0')
 		strcpy(fi->name, QR_get_value_manual(col_info->result, k, 3));
 
@@ -310,7 +307,7 @@ void lower_the_name(char *name, ConnectionClass *conn, BOOL dquote)
 		{
 			encoded_nextchar(&encstr);
 			if (ENCODE_STATUS(encstr) == 0)
-				*ptr = tolower((unsigned char) *ptr);
+				*ptr = tolower((UCHAR) *ptr);
 		}
 	}
 }
@@ -337,6 +334,7 @@ parse_statement(StatementClass *stmt)
 	char		in_field = FALSE,
 				in_expr = FALSE,
 				in_func = FALSE,
+                out_func = FALSE,
 				in_dot = FALSE,
 				in_as = FALSE;
 	int			j,
@@ -350,10 +348,14 @@ parse_statement(StatementClass *stmt)
 	ConnectionClass *conn = stmt->hdbc;
 	HSTMT		hcol_stmt;
 	StatementClass *col_stmt;
-	IRDFields	*irdflds = SC_get_IRD(stmt);
+	IRDFields	*irdflds = SC_get_IRDF(stmt);
 	RETCODE		result;
 	BOOL		updatable = TRUE;
 
+	QResultClass*	resultClass;
+	HSTMT			privStmt;
+	
+
 	mylog("%s: entering...\n", func);
 
 	ptr = stmt->statement;
@@ -680,14 +682,33 @@ parse_statement(StatementClass *stmt)
 					out_table = TRUE; 
 					continue;
 			}
-			if (out_table && !in_table) /* new table */
+			if (out_table && !in_table && !in_func) /* new table */
 			{
 				if (!dquote)
 				{
 					if (token[0] == '(' ||
 					    token[0] == ')')
 						continue;
+                    
+					/* 
+					 * Detect a function call that looks like a table, eg.
+					 *   SELECT * FROM version()
+					 * 
+					 * This needs work to properly handle functions found in the from
+					 * clause, but this at least prevents nasty errors for now.
+					 *
+					 * DJP, 2005-01-08
+					 */
+					if (ptr[0] == '(')
+					{
+                        in_func = TRUE;
+						mylog("**** got function = '%s'\n", token);
+						mylog("FIXME: functions in the FROM clause are currently ignored!!\n");
+						continue;
+					}
 				}
+                
+            
 				if (!(stmt->ntab % TAB_INCR))
 				{
 					ti = (TABLE_INFO **) realloc(ti, (stmt->ntab + TAB_INCR) * sizeof(TABLE_INFO *));
@@ -728,6 +749,64 @@ parse_statement(StatementClass *stmt)
 				continue;
 			}
 
+			if (out_table && !in_table && in_func) /* old function */
+			{
+                /* Just skipped a function alias */
+                if (out_func)
+                {
+                    in_func = FALSE;
+                    out_func = FALSE;
+                    mylog("leaving func (and ignoring alias)\n");
+                    continue;
+                }
+                
+                /* Normal function, no alias */
+                if (token[0] == ')' && (ptr[0] == ',' || strlen(ptr) == 0))
+                {
+                    in_func = FALSE;
+                    
+                    /* 
+                     * TODO: shouldn't we cache the metadata somewhere? 
+                     */
+                    SC_pre_execute(stmt);
+					resultClass = SC_get_Curres(stmt);
+
+					mylog("PGAPI_NumResultCols: result = %u, status = %d, numcols = %d\n", resultClass, stmt->status, resultClass!= NULL ? QR_NumResultCols(resultClass) : -1);
+					if ((!resultClass) || ((stmt->status != STMT_FINISHED) && (stmt->status != STMT_PREMATURE)))
+					{
+						/* no query has been executed on this statement */
+						SC_set_error(stmt, STMT_EXEC_ERROR, "No query has been executed with that handle");
+						SC_log_error(func, "", stmt);
+						return SQL_ERROR;
+					}
+					else if (!QR_command_maybe_successful(resultClass))
+					{
+						SC_set_errornumber(stmt, STMT_EXEC_ERROR);
+						SC_log_error(func, "", stmt);
+						return SQL_ERROR;
+					}
+					/* 
+					 * If the parse_statement is called to get the number of 
+					 * result columns throught SQLColAttribute. 
+					 */
+					irdflds->nfields = QR_NumPublicResultCols(resultClass);                    
+                    
+                    mylog("leaving func.\n");
+                    
+                    stmt->parse_status = STMT_PARSE_FATAL;
+					return	FALSE;
+                }
+                
+                /* Function with following alias */
+                if (token[0] == ')')
+                {
+                    out_func = TRUE;
+                    continue;
+                }
+                    
+                
+            }
+            
 			if (!dquote && stricmp(token, "JOIN") == 0)
 			{
 				in_table = FALSE;
