@@ -66,6 +66,7 @@ Int4 pgtypes_defined[]	= {
 				PG_TYPE_MONEY,
 				PG_TYPE_BOOL,
 				PG_TYPE_BYTEA,
+				PG_TYPE_NUMERIC,
 				PG_TYPE_LO_UNDEFINED,
 				0 };
 */
@@ -77,9 +78,7 @@ Int2		sqlTypes[] = {
 	/* SQL_BINARY, -- Commented out because VarBinary is more correct. */
 	SQL_BIT,
 	SQL_CHAR,
-#if (ODBCVER >= 0x0300)
 	SQL_TYPE_DATE,
-#endif /* ODBCVER */
 	SQL_DATE,
 	SQL_DECIMAL,
 	SQL_DOUBLE,
@@ -90,29 +89,25 @@ Int2		sqlTypes[] = {
 	SQL_NUMERIC,
 	SQL_REAL,
 	SQL_SMALLINT,
-#if (ODBCVER >= 0x0300)
 	SQL_TYPE_TIME,
 	SQL_TYPE_TIMESTAMP,
-#endif /* ODBCVER */
 	SQL_TIME,
 	SQL_TIMESTAMP,
 	SQL_TINYINT,
 	SQL_VARBINARY,
 	SQL_VARCHAR,
-#ifdef	UNICODE_SUPPORT
 	SQL_WCHAR,
 	SQL_WVARCHAR,
 	SQL_WLONGVARCHAR,
-#endif /* UNICODE_SUPPORT */
 	0
 };
 
-#if (ODBCVER >= 0x0300) && defined(ODBCINT64)
+#ifdef ODBCINT64
 #define	ALLOWED_C_BIGINT	SQL_C_SBIGINT
 /* #define	ALLOWED_C_BIGINT	SQL_C_CHAR */ /* Delphi should be either ? */
 #else
 #define	ALLOWED_C_BIGINT	SQL_C_CHAR
-#endif
+#endif /* ODBCINT64 */
 
 Int4
 sqltype_to_pgtype(StatementClass *stmt, SWORD fSqlType)
@@ -131,21 +126,15 @@ sqltype_to_pgtype(StatementClass *stmt, SWORD fSqlType)
 			pgType = PG_TYPE_BPCHAR;
 			break;
 
-#ifdef	UNICODE_SUPPORT
 		case SQL_WCHAR:
 			pgType = PG_TYPE_BPCHAR;
 			break;
-#endif /* UNICODE_SUPPORT */
 
 		case SQL_BIT:
 			pgType = ci->drivers.bools_as_char ? PG_TYPE_CHAR : PG_TYPE_BOOL;
 			break;
 
-#if (ODBCVER >= 0x0300)
 		case SQL_TYPE_DATE:
-#else
-		case SQL_DATE:
-#endif /* ODBCVER */
 			pgType = PG_TYPE_DATE;
 			break;
 
@@ -178,11 +167,9 @@ sqltype_to_pgtype(StatementClass *stmt, SWORD fSqlType)
 			pgType = ci->drivers.text_as_longvarchar ? PG_TYPE_TEXT : PG_TYPE_VARCHAR;
 			break;
 
-#ifdef	UNICODE_SUPPORT
 		case SQL_WLONGVARCHAR:
 			pgType = ci->drivers.text_as_longvarchar ? PG_TYPE_TEXT : PG_TYPE_VARCHAR;
 			break;
-#endif /* UNICODE_SUPPORT */
 
 		case SQL_REAL:
 			pgType = PG_TYPE_FLOAT4;
@@ -194,16 +181,12 @@ sqltype_to_pgtype(StatementClass *stmt, SWORD fSqlType)
 			break;
 
 		case SQL_TIME:
-#if (ODBCVER >= 0x0300)
 		case SQL_TYPE_TIME:
-#endif /* ODBCVER */
 			pgType = PG_TYPE_TIME;
 			break;
 
 		case SQL_TIMESTAMP:
-#if (ODBCVER >= 0x0300)
 		case SQL_TYPE_TIMESTAMP:
-#endif /* ODBCVER */
 			pgType = PG_TYPE_DATETIME;
 			break;
 
@@ -215,11 +198,9 @@ sqltype_to_pgtype(StatementClass *stmt, SWORD fSqlType)
 			pgType = PG_TYPE_VARCHAR;
 			break;
 
-#if	UNICODE_SUPPORT
 		case SQL_WVARCHAR:
 			pgType = PG_TYPE_VARCHAR;
 			break;
-#endif /* UNICODE_SUPPORT */
 
 		default:
 			pgType = 0;			/* ??? */
@@ -243,13 +224,12 @@ sqltype_to_pgtype(StatementClass *stmt, SWORD fSqlType)
  *	types that are unknown.  All other pg routines in here return a suitable default.
  */
 Int2
-pgtype_to_concise_type(StatementClass *stmt, Int4 type)
+pgtype_to_concise_type(StatementClass *stmt, Int4 type, int col)
 {
 	ConnectionClass	*conn = SC_get_conn(stmt);
 	ConnInfo	*ci = &(conn->connInfo);
-#if (ODBCVER >= 0x0300)
+
 	EnvironmentClass *env = (EnvironmentClass *) (conn->henv);
-#endif /* ODBCVER */
 
 	switch (type)
 	{
@@ -260,11 +240,16 @@ pgtype_to_concise_type(StatementClass *stmt, Int4 type)
 		case PG_TYPE_NAME:
 			return SQL_CHAR;
 
-#ifdef	UNICODE_SUPPORT
 		case PG_TYPE_BPCHAR:
+			if (col >= 0 &&
+			    getCharColumnSize(stmt, type, col, UNKNOWNS_AS_MAX) > ci->drivers.max_varchar_size)
+				return conn->unicode ? SQL_WLONGVARCHAR : SQL_LONGVARCHAR;
 			return conn->unicode ? SQL_WCHAR : SQL_CHAR;
 
 		case PG_TYPE_VARCHAR:
+			if (col >= 0 &&
+			    getCharColumnSize(stmt, type, col, UNKNOWNS_AS_MAX) > ci->drivers.max_varchar_size)
+				return conn->unicode ? SQL_WLONGVARCHAR : SQL_LONGVARCHAR;
 			return conn->unicode ? SQL_WVARCHAR : SQL_VARCHAR;
 
 		case PG_TYPE_TEXT:
@@ -272,17 +257,6 @@ pgtype_to_concise_type(StatementClass *stmt, Int4 type)
 				(conn->unicode ? SQL_WLONGVARCHAR : SQL_LONGVARCHAR) :
 				(conn->unicode ? SQL_WVARCHAR : SQL_VARCHAR);
 
-#else
-		case PG_TYPE_BPCHAR:
-			return SQL_CHAR;
-
-		case PG_TYPE_VARCHAR:
-			return SQL_VARCHAR;
-
-		case PG_TYPE_TEXT:
-			return ci->drivers.text_as_longvarchar ? SQL_LONGVARCHAR : SQL_VARCHAR;
-#endif /* UNICODE_SUPPORT */
-
 		case PG_TYPE_BYTEA:
 			if (ci->bytea_as_longvarbinary)
 				return SQL_LONGVARBINARY;
@@ -305,11 +279,8 @@ pgtype_to_concise_type(StatementClass *stmt, Int4 type)
 				return ci->int8_as;
 			if (conn->ms_jet) 
 				return SQL_NUMERIC; /* maybe a little better than SQL_VARCHAR */
-#if (ODBCVER >= 0x0300)
+			
 			return SQL_BIGINT;
-#else
-			return SQL_VARCHAR;
-#endif /* ODBCVER */
 
 		case PG_TYPE_NUMERIC:
 			return SQL_NUMERIC;
@@ -319,25 +290,19 @@ pgtype_to_concise_type(StatementClass *stmt, Int4 type)
 		case PG_TYPE_FLOAT8:
 			return SQL_FLOAT;
 		case PG_TYPE_DATE:
-#if (ODBCVER >= 0x0300)
 			if (EN_is_odbc3(env))
 				return SQL_TYPE_DATE;
-#endif /* ODBCVER */
 			return SQL_DATE;
 		case PG_TYPE_TIME:
-#if (ODBCVER >= 0x0300)
 			if (EN_is_odbc3(env))
 				return SQL_TYPE_TIME;
-#endif /* ODBCVER */
 			return SQL_TIME;
 		case PG_TYPE_ABSTIME:
 		case PG_TYPE_DATETIME:
 		case PG_TYPE_TIMESTAMP_NO_TMZONE:
 		case PG_TYPE_TIMESTAMP:
-#if (ODBCVER >= 0x0300)
 			if (EN_is_odbc3(env))
 				return SQL_TYPE_TIMESTAMP;
-#endif /* ODBCVER */
 			return SQL_TIMESTAMP;
 		case PG_TYPE_MONEY:
 			return SQL_FLOAT;
@@ -360,18 +325,16 @@ pgtype_to_concise_type(StatementClass *stmt, Int4 type)
 }
 
 Int2
-pgtype_to_sqldesctype(StatementClass *stmt, Int4 type)
+pgtype_to_sqldesctype(StatementClass *stmt, Int4 type, int col)
 {
 	Int2	rettype;
 
-	switch (rettype = pgtype_to_concise_type(stmt, type))
+	switch (rettype = pgtype_to_concise_type(stmt, type, col))
 	{
-#if (ODBCVER >= 0x0300)
 		case SQL_TYPE_DATE:
 		case SQL_TYPE_TIME:
 		case SQL_TYPE_TIMESTAMP:
 			return SQL_DATETIME;
-#endif /* ODBCVER */
 	}
 	return rettype;
 }
@@ -379,16 +342,14 @@ pgtype_to_sqldesctype(StatementClass *stmt, Int4 type)
 Int2
 pgtype_to_datetime_sub(StatementClass *stmt, Int4 type)
 {
-	switch (pgtype_to_concise_type(stmt, type))
+	switch (pgtype_to_concise_type(stmt, type, PG_STATIC))
 	{
-#if (ODBCVER >= 0x0300)
 		case SQL_TYPE_DATE:
 			return SQL_CODE_DATE;
 		case SQL_TYPE_TIME:
 			return SQL_CODE_TIME;
 		case SQL_TYPE_TIMESTAMP:
 			return SQL_CODE_TIMESTAMP;
-#endif /* ODBCVER */
 	}
 	return -1;
 }
@@ -399,17 +360,15 @@ pgtype_to_ctype(StatementClass *stmt, Int4 type)
 {
 	ConnectionClass	*conn = SC_get_conn(stmt);
 	ConnInfo	*ci = &(conn->connInfo);
-#if (ODBCVER >= 0x0300)
+
 	EnvironmentClass *env = (EnvironmentClass *) (conn->henv);
-#endif /* ODBCVER */
+
 
 	switch (type)
 	{
 		case PG_TYPE_INT8:
-#if (ODBCVER >= 0x0300)
 			if (!conn->ms_jet)
 				return ALLOWED_C_BIGINT;
-#endif /* ODBCVER */
 			return SQL_C_CHAR;
 		case PG_TYPE_NUMERIC:
 			return SQL_C_CHAR;
@@ -424,25 +383,19 @@ pgtype_to_ctype(StatementClass *stmt, Int4 type)
 		case PG_TYPE_FLOAT8:
 			return SQL_C_DOUBLE;
 		case PG_TYPE_DATE:
-#if (ODBCVER >= 0x0300)
 			if (EN_is_odbc3(env))
 				return SQL_C_TYPE_DATE;
-#endif /* ODBCVER */
 			return SQL_C_DATE;
 		case PG_TYPE_TIME:
-#if (ODBCVER >= 0x0300)
 			if (EN_is_odbc3(env))
 				return SQL_C_TYPE_TIME;
-#endif /* ODBCVER */
 			return SQL_C_TIME;
 		case PG_TYPE_ABSTIME:
 		case PG_TYPE_DATETIME:
 		case PG_TYPE_TIMESTAMP_NO_TMZONE:
 		case PG_TYPE_TIMESTAMP:
-#if (ODBCVER >= 0x0300)
 			if (EN_is_odbc3(env))
 				return SQL_C_TYPE_TIMESTAMP;
-#endif /* ODBCVER */
 			return SQL_C_TIMESTAMP;
 		case PG_TYPE_MONEY:
 			return SQL_C_FLOAT;
@@ -453,14 +406,12 @@ pgtype_to_ctype(StatementClass *stmt, Int4 type)
 			return SQL_C_BINARY;
 		case PG_TYPE_LO_UNDEFINED:
 			return SQL_C_BINARY;
-#ifdef	UNICODE_SUPPORT
 		case PG_TYPE_BPCHAR:
 		case PG_TYPE_VARCHAR:
 		case PG_TYPE_TEXT:
 			if (conn->unicode && ! conn->ms_jet && ! stmt->manual_result)
 				return SQL_C_WCHAR;
 			return SQL_C_CHAR;
-#endif /* UNICODE_SUPPORT */
 
 		default:
 			/* hack until permanent type is available */
@@ -515,10 +466,12 @@ pgtype_to_name(StatementClass *stmt, Int4 type)
 		case PG_TYPE_ABSTIME:
 			return "abstime";
 		case PG_TYPE_DATETIME:
-			if (PG_VERSION_GE(conn, 7.0))
-				return "timestamp with time zone";
-			else
+			if (PG_VERSION_GT(conn, 7.1))
+				return "timestamptz";
+			else if (PG_VERSION_LT(conn, 7.0))
 				return "datetime";
+			else
+				return "timestamp";
 		case PG_TYPE_TIMESTAMP_NO_TMZONE:
 			return "timestamp without time zone";
 		case PG_TYPE_TIMESTAMP:
@@ -689,7 +642,8 @@ getCharColumnSize(StatementClass *stmt, Int4 type, int col, int handle_unknown_s
 	if (col < 0)
 		return maxsize;
 
-	result = SC_get_Curres(stmt);
+	if (result = SC_get_Curres(stmt), NULL == result)
+		return maxsize;
 
 	/*
 	 * Manual Result Sets -- use assigned column width (i.e., from
@@ -717,10 +671,7 @@ getCharColumnSize(StatementClass *stmt, Int4 type, int col, int handle_unknown_s
 			case PG_TYPE_BPCHAR:
 				if (conn->unicode || conn->ms_jet)
 					return attlen;
-#if (ODBCVER >= 0x0300)
 				return attlen;
-#endif /* ODBCVER */
-				return p;
 		}
 	}
 
@@ -740,6 +691,16 @@ getCharColumnSize(StatementClass *stmt, Int4 type, int col, int handle_unknown_s
 		return -1;
 }
 
+static Int2
+getTimestampMaxDecimalDigits(StatementClass *stmt, Int4 type)
+{
+	ConnectionClass *conn = SC_get_conn(stmt);
+
+	if (PG_VERSION_LT(conn, 7.2))
+		return 0;
+	return 6;
+}
+
 static Int2
 getTimestampDecimalDigits(StatementClass *stmt, Int4 type, int col)
 {
@@ -1107,6 +1068,64 @@ pgtype_transfer_octet_length(StatementClass *stmt, Int4 type, int col, int handl
 	return -1;
 }
 
+/*
+ *	corrsponds to "min_scale" in ODBC 2.x.
+ */
+Int2
+pgtype_min_decimal_digits(StatementClass *stmt, Int4 type)
+{
+	switch (type)
+	{
+		case PG_TYPE_INT2:
+		case PG_TYPE_OID:
+		case PG_TYPE_XID:
+		case PG_TYPE_INT4:
+		case PG_TYPE_INT8:
+		case PG_TYPE_FLOAT4:
+		case PG_TYPE_FLOAT8:
+		case PG_TYPE_MONEY:
+		case PG_TYPE_BOOL:
+		case PG_TYPE_ABSTIME:
+		case PG_TYPE_TIMESTAMP:
+		case PG_TYPE_DATETIME:
+		case PG_TYPE_TIMESTAMP_NO_TMZONE:
+		case PG_TYPE_NUMERIC:
+			return 0;
+		default:
+			return -1;
+	}
+}
+
+/*
+ *	corrsponds to "max_scale" in ODBC 2.x.
+ */
+Int2
+pgtype_max_decimal_digits(StatementClass *stmt, Int4 type)
+{
+	switch (type)
+	{
+		case PG_TYPE_INT2:
+		case PG_TYPE_OID:
+		case PG_TYPE_XID:
+		case PG_TYPE_INT4:
+		case PG_TYPE_INT8:
+		case PG_TYPE_FLOAT4:
+		case PG_TYPE_FLOAT8:
+		case PG_TYPE_MONEY:
+		case PG_TYPE_BOOL:
+		case PG_TYPE_ABSTIME:
+		case PG_TYPE_TIMESTAMP:
+			return 0;
+		case PG_TYPE_DATETIME:
+		case PG_TYPE_TIMESTAMP_NO_TMZONE:
+			return getTimestampMaxDecimalDigits(stmt, type);
+		case PG_TYPE_NUMERIC:
+			return 38;
+		default:
+			return -1;
+	}
+}
+
 /*
  *	corrsponds to "scale" in ODBC 2.x.
  */
@@ -1352,6 +1371,8 @@ pgtype_create_params(StatementClass *stmt, Int4 type)
 		case PG_TYPE_BPCHAR:
 		case PG_TYPE_VARCHAR:
 			return "max. length";
+		case PG_TYPE_NUMERIC:
+			return "precision, scale";
 		default:
 			return NULL;
 	}
@@ -1372,23 +1393,15 @@ sqltype_to_default_ctype(const ConnectionClass *conn, Int2 sqltype)
 		case SQL_LONGVARCHAR:
 		case SQL_DECIMAL:
 		case SQL_NUMERIC:
-#if (ODBCVER < 0x0300)
-		case SQL_BIGINT:
-			return SQL_C_CHAR;
-#else
 			return SQL_C_CHAR;
 		case SQL_BIGINT:
 			return ALLOWED_C_BIGINT;
-#endif
-
-#ifdef	UNICODE_SUPPORT
 		case SQL_WCHAR:
 		case SQL_WVARCHAR:
 		case SQL_WLONGVARCHAR:
 			if (conn->ms_jet || ! conn->unicode)
 				return SQL_C_CHAR;
 			return SQL_C_WCHAR;
-#endif /* UNICODE_SUPPORT */
 
 		case SQL_BIT:
 			return SQL_C_BIT;
@@ -1423,7 +1436,6 @@ sqltype_to_default_ctype(const ConnectionClass *conn, Int2 sqltype)
 		case SQL_TIMESTAMP:
 			return SQL_C_TIMESTAMP;
 
-#if (ODBCVER >= 0x0300)
 		case SQL_TYPE_DATE:
 			return SQL_C_TYPE_DATE;
 
@@ -1432,7 +1444,6 @@ sqltype_to_default_ctype(const ConnectionClass *conn, Int2 sqltype)
 
 		case SQL_TYPE_TIMESTAMP:
 			return SQL_C_TYPE_TIMESTAMP;
-#endif /* ODBCVER */
 
 		default:
 			/* should never happen */
@@ -1476,28 +1487,20 @@ ctype_length(Int2 ctype)
 			return sizeof(UCHAR);
 
 		case SQL_C_DATE:
-#if (ODBCVER >= 0x0300)
 		case SQL_C_TYPE_DATE:
-#endif /* ODBCVER */
 			return sizeof(DATE_STRUCT);
 
 		case SQL_C_TIME:
-#if (ODBCVER >= 0x0300)
 		case SQL_C_TYPE_TIME:
-#endif /* ODBCVER */
 			return sizeof(TIME_STRUCT);
 
 		case SQL_C_TIMESTAMP:
-#if (ODBCVER >= 0x0300)
 		case SQL_C_TYPE_TIMESTAMP:
-#endif /* ODBCVER */
 			return sizeof(TIMESTAMP_STRUCT);
 
 		case SQL_C_BINARY:
 		case SQL_C_CHAR:
-#ifdef	UNICODE_SUPPORT
 		case SQL_C_WCHAR:
-#endif /* UNICODE_SUPPORT */
 			return 0;
 
 		default:				/* should never happen */
